<?php
namespace GopNext\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class BudgetPreparationService {
    
    /**
    * Class constructor
    * 
    * Initialize an object's properties upon creation of the object.
    * 
    * @param object $connection - Establishes the connection to the database.
    * 
    * @return void
    * 
    * @access public
    * 
    * @author Grazielle Ferreira Antunes
    */
    public function __construct($connection){
        $this->connection   = $connection;
    }

    /**
     * Establishes the connection to the database and executes the query to return all units.
     * 
     * @param object $params – Object that contains the parameters necessary to execute the query.
     * 
     * @return array - Array with all units that meet the parameters.
     * 
     * @access public
     * 
     * @author Grazielle Ferreira Antunes
     */ 
    public function selectUnit($params) {
        $select = "SELECT
                        '' AS IDMARCADO,
                        FL.CDFILIAL,
                        FL.NMFILIAL,
                        FL.NRORG
                    FROM
                        FILIAL         FL,
                        FILGRUPFILI    FG,
                        FILIALOPEROP   FO
                    WHERE
                        ( FL.CDFILIAL = FG.CDFILIAL (+) )
                        AND ( FO.CDFILIAL = FL.CDFILIAL )
                        AND ( ( INSTR(:CDGRUPFILI, FG.CDGRUPFILI) > 0 )
                              OR ( 'T' = NVL(:CDGRUPFILI, 'T') ) )
                        AND ( ( INSTR(:CDSEGMER, FL.CDSEGMER) > 0 )
                              OR ( 'T' = NVL(:CDSEGMER, 'T') ) )
                        AND ( ( INSTR(:CDOPERADOR, FO.CDOPERADOR) > 0 )
                              OR ( 'T' = NVL(:CDOPERADOR, 'T') ) )
                        AND ( (FL.CDSUPERVISOR = :CDSUPERVISOR)
                              OR ( 'T' = NVL(:CDSUPERVISOR, 'T') ) )
                        AND ( (FL.CDREGIAO = :CDREGIAO ) 
                              OR ( 'T' = NVL(:CDREGIAO, 'T') ) )
                    GROUP BY
                        FL.CDFILIAL,
                        FL.NMFILIAL,
                        FL.NRORG
                    ORDER BY
                        FL.CDFILIAL";
                        
        $std = $this->connection->prepare($select);
        $std->execute($params);
        $result = $std->fetchAll();
        return $result;
    }

    /**
    * Establishes the connection to the database and executes the query to return all errors from the log table after executing the procedure.
    * 
    * @param object $params – Object that contains the parameters necessary to execute the query.
    * 
    * @return array - Array with the list of errors that were generated when executing the procedure.
    * 
    * @access public
    * 
    * @author Grazielle Ferreira Antunes
    */  
    public function noGenerated($params){
        $select = "SELECT F.CDFILIAL, F.NMFILIAL, L.DSMOTIVO FROM LOGPREPARAORCAGOP L, FILIAL F, LISTA LF
             WHERE
               LF.NRLISTA = :NRLISTA AND
               LF.CDLISTA = F.CDFILIAL AND
               L.NRANOORCAGOP = :NRANOORCAGOP AND
               L.NRPERIODOORCA = :NRPERIODOORCA AND
               L.IDPERIODOORCA = :IDPERIODOORCA
             ORDER BY F.CDFILIAL";
        $stmt = $this->connection->prepare($select); 
        $stmt->execute($params);
        $result = $stmt->fetchAll();
		return  $result;
    }
    
    /**
    * Establishes the connection with the database and executes the procedure that prepares the budget.
    * 
    * @param object $params – Object that contains the parameters necessary to execute the procedure.
    * @param string $nrListaOperador – Number of the list generated to store operators.
    * @param string $nrListaUnidade – Number of the list generated to store units.
    *  @param string $cdOperLogado – Logged in operator code.
    * 
    * @return array - Array with the list of errors that were generated when executing the procedure.
    * 
    * @access public
    * 
    * @author Grazielle Ferreira Antunes
    */  
    public function PREPARA_ORCAMENTO($row, $nrListaOperador, $nrListaUnidade, $cdOperLogado)
    {
        $procedure = new StoredProcedure($this->connection, 'PREPARA_ORCAMENTO');
        $procedure->addParam(new Param('P_NRANOORCAOP', Param::PARAM_INPUT, $row['NRANOORCAGOP'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDVERSORCAOP', Param::PARAM_INPUT, $row['VALUE_PERIOD'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $nrListaOperador, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $nrListaUnidade, Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDEXCLUI', Param::PARAM_INPUT, $row['IDEXCLUIR'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTAOPER', Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_LISTAFIL', Param::PARAM_INPUT, 'S', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ORCADETA', Param::PARAM_INPUT, 'X', Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ITEMORCA', Param::PARAM_INPUT, $row['STRINGTPORCA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERLOGADO', Param::PARAM_INPUT, $cdOperLogado, Param::PARAM_TYPE_STR));
    
        $procedure->execute();
    }
}