<?php
namespace GopNext\Services;

use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class PercentageAndStructureForBudgetService{
    public function __construct($entityManager, $environment){
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
    }
    
    public function beginTransaction() {
        $this->connection->beginTransaction();
    }
    
    public function commit() {
        $this->connection->commit();
    }
    
    public function rollback() {
        $this->connection->rollback();
    }
    
    public function selectItemsOperational() {
        $query = "SELECT NRSEQITEMORCA FROM PERCITEMGOP";
        try {
            $std = $this->connection->prepare($query);
            $std->execute();
            $result = $std->fetchAll();
            return $result;
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function selectItemsOperationalUnit($params) {
        $query = "SELECT NRSEQITEMORCA FROM PERCITFILGOP WHERE CDFILIAL = :CDFILIAL";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
            $result = $std->fetchAll();
            return $result;
        } catch (Exception $e){
            throw $e;
        };
    }
      
    public function insertItemsOperational($params) {
        $query = "INSERT INTO PERCITEMGOP (NRPERCITEMGOP, NRSEQITEMORCA, DSITEMORCAMENTO, VRPERCITEMGOPM, NRORG, IDATIVO) VALUES (:NRPERCITEMGOP, :NRSEQITEMORCA, :DSITEMORCAMENTO, :VRPERCITEMGOP, :NRORG, 'S')";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function insertItemsOperationalUnit($params) {
        $query = "INSERT INTO PERCITFILGOP (NRPERCITFILGOP, NRSEQITEMORCA, DSITEMORCAMENTO, VRPERCITEMGOP, CDFILIAL, NRORG, IDATIVO) VALUES (SEQ_PERCITFILGOP.NEXTVAL, :NRSEQITEMORCA, :DSITEMORCAMENTO, :VRPERCITEMGOP, :CDFILIAL, :NRORG, 'S' )";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }

    public function updateOperationalItem($params, $seq) {
        $query = "";
        
        if ($params['CDFILIAL'] === 'P') {
            unset($params["CDFILIAL"]);
            $query = "UPDATE PERCITEMGOP
                         SET VRPERCITEMGOP = :VRPERCITEMGOP,
                             CDOPERULTATU = :CDOPERADOR,
                             DTULTATU = SYSDATE
                       WHERE NRSEQITEMORCA = :NRSEQITEMORCA
                         AND NRORG = :NRORG";
        } else {
            if ($seq) {
                $query = "UPDATE PERCITFILGOP
                             SET VRPERCITEMGOP = :VRPERCITEMGOP,
                                 CDOPERULTATU = :CDOPERADOR,
                                 DTULTATU = SYSDATE
                           WHERE CDFILIAL = :CDFILIAL
                             AND NRSEQITEMORCA = :NRSEQITEMORCA
                             AND NRORG = :NRORG";
            } else {
                $query = "INSERT INTO PERCITFILGOP (NRPERCITFILGOP, CDFILIAL, NRSEQITEMORCA, DSITEMORCAMENTO, VRPERCITEMGOP,
                                                    DTINCLUSAO, CDOPERINCLUSAO, NRORG)
                               SELECT SEQ_PERCITFILGOP.NEXTVAL, :CDFILIAL, NRSEQITEMORCA, DSITEMORCAMENTO, :VRPERCITEMGOP,
                                      SYSDATE, :CDOPERADOR, NRORG
                                 FROM PERCITEMGOP
                                WHERE NRSEQITEMORCA = :NRSEQITEMORCA
                                  AND NRORG = :NRORG";
            }
        }

        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function insertDetail($params) {
        $query = "INSERT INTO PERCITEMDETALGOP (
                    NRPERCITEMGOP, NRPERCITEMDETALGOP, CDPRODUTO, CDIMPOSTO, CDTIPOOPER, IDENTRSAIDOP, CDINVEST, CDDESPADMFAT,
                    CDDESPPREV, CDFILIALMO, VRPERCENTUAL, IDATIVO, DTINCLUSAO, CDOPERINCLUSAO,
                    CDORCAMENTOMO, CDFATUDIVER, CDSUBGRPROD, CDSERVFILI, CDTPCTPAGAR, CDCONTCTBL, CDGRUPPROD, DSEXPRESSAO
                ) VALUES (
                    :NRPERCITEMGOP, SEQ_PERCITEMDETALGOP.NEXTVAL, :CDPRODUTO, :CDIMPOSTO, :CDTIPOOPER, :IDENTRSAIDOP, :CDINVEST, 
                    :CDDESPADMFAT, :CDDESPPREV, :CDFILIALMO, :VRPERCENTUAL, 'S', SYSDATE,
                    :CDOPERINCLUSAO, :CDORCAMENTOMO,:CDFATUDIVER, :CDSUBGRPROD, :CDSERVFILI, :CDTPCTPAGAR, :CDCONTCTBL,
                    :CDGRUPPROD, :DSEXPRESSAO
                )";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function insertDetailUnit($params) {
        $query = "INSERT INTO PERCITDETALFILGOP (
                    NRPERCITFILGOP, NRPERCITDETALFILGOP, CDFILIAL, CDPRODUTO, CDIMPOSTO, CDTIPOOPER, CDFILIALMO, IDENTRSAIDOP,
                    CDINVEST, CDDESPADMFAT, CDDESPPREV, IDDESPPREV, VRPERCENTUAL, NRORG, IDATIVO, DTINCLUSAO, 
                    NRORGINCLUSAO, CDOPERINCLUSAO, CDORCAMENTOMO, CDFATUDIVER, CDSERVFILI, CDSUBGRPROD, CDTPCTPAGAR, CDCONTCTBL, 
                    CDGRUPPROD, DSEXPRESSAO
                ) VALUES (
                    :NRPERCITFILGOP, SEQ_PERCITDETALFILGOP.NEXTVAL, :CDFILIAL, :CDPRODUTO, :CDIMPOSTO, :CDTIPOOPER, :CDFILIALMO, 
                    :IDENTRSAIDOP, :CDINVEST, :CDDESPADMFAT, :CDDESPPREV, :IDDESPPREV, :VRPERCENTUAL, :NRORG,
                    'S', SYSDATE, :NRORGINCLUSAO, :CDOPERINCLUSAO, :CDORCAMENTOMO, :CDFATUDIVER, :CDSERVFILI, :CDSUBGRPROD,
                    :CDTPCTPAGAR, :CDCONTCTBL, :CDGRUPPROD, :DSEXPRESSAO
                )";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function updateDetail($params) {
         $query = "UPDATE PERCITEMDETALGOP SET VRPERCENTUAL = :VRPERCENTUAL WHERE NRPERCITEMDETALGOP = :NRPERCITEMDETALGOP";
         try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function updateDetailUnit($params) {
        $query = "UPDATE PERCITDETALFILGOP SET VRPERCENTUAL = :VRPERCENTUAL WHERE NRPERCIDETALFILGOP = :NRPERCIDETALFILGOP";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function updateDetailModal($params) {
        $query = "UPDATE PERCDETALPRODGOP SET VRPERCENTUAL = :VRPERCENTUAL WHERE NRPERCDETALPRODGOP = :NRPERCDETALPRODGOP";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function insertDetailModal($params) {
        $query = "INSERT INTO PERCDETALPRODGOP (NRPERCITEMDETALGOP, NRPERCDETALPRODGOP, CDPRODUTO, CDORCAMENTOMO, CDDESPADMFAT, 
                    CDTIPOOPER, CDFILIALMO, IDENTRSAIDOP, VRPERCENTUAL, NRORG, IDATIVO, DTINCLUSAO, CDCONTCTBL, CDTPCTPAGAR) 
                    VALUES (:NRPERCITEMDETALGOP, SEQ_PERCDETALPRODGOP.NEXTVAL, :CDPRODUTO, :CDORCAMENTOMO, :CDDESPADMFAT, :CDTIPOOPER, 
                    :CDFILIALMO, :IDENTRSAIDOP, :VRPERCENTUAL, :NRORG, 'S', SYSDATE, :CDCONTCTBL, :CDTPCTPAGAR)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function updateDetailModalUnit($params) {
        $query = "UPDATE PERCDETPRODFILGOP SET VRPERCENTUAL = :VRPERCENTUAL WHERE NRPERCDETPRODFILGOP = :NRPERCDETPRODFILGOP";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function insertDetailModalUnit($params) {
        $query = "INSERT INTO PERCDETPRODFILGOP (NRPERCITEMDETALFILGOP, NRPERCDETPRODFILGOP, CDFILIAL, CDPRODUTO, CDORCAMENTOMO, 
                    CDDESPADMFAT, CDTIPOOPER, CDFILIALMO, IDENTRSAIDOP, VRPERCENTUAL, NRORG, IDATIVO, DTINCLUSAO, CDCONTCTBL, CDTPCTPAGAR) 
                    VALUES (:NRPERCITEMDETALGOP, SEQ_PERCDETPRODFILGOP.NEXTVAL, :CDPRODUTO, :CDORCAMENTOMO, :CDDESPADMFAT, 
                    :CDTIPOOPER, :CDFILIALMO, :IDENTRSAIDOP, :VRPERCENTUAL, :NRORG, 'S', SYSDATE, :CDCONTCTBL, :CDTPCTPAGAR)";
        try {
            $std = $this->connection->prepare($query);
            $std->execute($params);
        } catch (Exception $e){
            throw $e;
        };
    }
    
    public function executeScriptSequence($tableSequence) {
        $script = "DECLARE V_NOVOCODIGO NUMBER;
            BEGIN
               V_NOVOCODIGO := NULL;
               BEGIN
                  SELECT TO_NUMBER(NRSEQUENCIAL)+1
                    INTO V_NOVOCODIGO
                    FROM NOVOCODIGO
                   WHERE CDCONTADOR = '$tableSequence';
               EXCEPTION
                  WHEN NO_DATA_FOUND THEN
                     V_NOVOCODIGO := 1;
                  WHEN VALUE_ERROR THEN
                     DBMS_OUTPUT.PUT_LINE('O campo não contém apenas números.');
               END;
               IF V_NOVOCODIGO IS NOT NULL THEN
                  BEGIN
                     EXECUTE IMMEDIATE 'CREATE SEQUENCE SEQ_$tableSequence MINVALUE '||V_NOVOCODIGO||' INCREMENT BY 1 START WITH '||V_NOVOCODIGO||' CACHE 100 NOCYCLE';
                     DBMS_OUTPUT.PUT_LINE('Sequência SEQ_$tableSequence criada com sucesso.');
                  EXCEPTION
                     WHEN OTHERS THEN
                        IF SQLCODE = -955 THEN
                           DBMS_OUTPUT.PUT_LINE('A sequência SEQ_$tableSequence já existe.');
                        ELSE
                           DBMS_OUTPUT.PUT_LINE('Erro ao criar a sequência SEQ_$tableSequence: ' || SQLERRM);
                        END IF;
                  END;
               END IF;
            END;";
        $stmt = $this->connection->prepare($script);
        $stmt->execute();
    }
    
    public function replicateImportParams($params) {
        $procedure = new StoredProcedure($this->connection, 'COPYPARAMS_GOP');
        $procedure->addParam(new Param('P_CDFILDEST', Param::PARAM_INPUT, $params['CDFILDEST'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_ITEMOPERA', Param::PARAM_INPUT, $params['ITEMOPERA'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDOPERADOR', Param::PARAM_INPUT, $params['CDOPERADOR'], Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDFILORIG', Param::PARAM_INPUT, $params['CDFILORIG'], Param::PARAM_TYPE_STR));
        
        $procedure->execute();
    }
}