import { I18n } from '@zeedhi/core';
import {
  AlertService,
  DialogService,
  Modal,
  ModalService,
} from '@zeedhi/common';
import BudgetParameters from '@/controllers/BudgetParameters';

export class Utilities {
  /**
  * Defines the size of the period field.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public periodSize = 10;
  
  /**
  * Lists existing budget types.
  *
  * @access public
  * @author Grazielle Antunes
  */
  periodList = [
    {
      NRPERIODOORCA: 1,
      DSPERIODOORCA: I18n.translate('gop::PRIMEIROSEMESTRE'),
      IDPERIODOORCA: 'SEMESTRAL',
    },
    {
      NRPERIODOORCA: 2,
      DSPERIODOORCA: I18n.translate('gop:SEGUNDOSEMESTRE'),
      IDPERIODOORCA: 'SEMESTRAL',
    },
    {
      NRPERIODOORCA: 1,
      DSPERIODOORCA: I18n.translate('gop:PRIMEIROQUADRIMESTRE'),
      IDPERIODOORCA: 'QUADRIMESTRAL',
    },
    {
      NRPERIODOORCA: 2,
      DSPERIODOORCA: I18n.translate('gop:SEGUNDOQUADRIMESTRE'),
      IDPERIODOORCA: 'QUADRIMESTRAL',
    },
    {
      NRPERIODOORCA: 3,
      DSPERIODOORCA: I18n.translate('gop:TERCEIROQUADRIMESTRE'),
      IDPERIODOORCA: 'QUADRIMESTRAL',
    },
    {
      NRPERIODOORCA: 1,
      DSPERIODOORCA: I18n.translate('gop:PRIMEIROTRIMESTRE'),
      IDPERIODOORCA: 'TRIMESTRAL',
    },
    {
      NRPERIODOORCA: 2,
      DSPERIODOORCA: I18n.translate('gop:SEGUNDOTRIMESTRE'),
      IDPERIODOORCA: 'TRIMESTRAL',
    },
    {
      NRPERIODOORCA: 3,
      DSPERIODOORCA: I18n.translate('gop:TERCEIROTRIMESTRE'),
      IDPERIODOORCA: 'TRIMESTRAL',
    },
    {
      NRPERIODOORCA: 4,
      DSPERIODOORCA: I18n.translate('gop:QUARTOTRIMESTRE'),
      IDPERIODOORCA: 'TRIMESTRAL',
    },
    {
      NRPERIODOORCA: 1,
      DSPERIODOORCA: I18n.translate('gop:PRIMEIROBIMESTRE'),
      IDPERIODOORCA: 'BIMESTRAL',
    },
    {
      NRPERIODOORCA: 2,
      DSPERIODOORCA: I18n.translate('gop:SEGUNDOBIMESTRE'),
      IDPERIODOORCA: 'BIMESTRAL',
    },
    {
      NRPERIODOORCA: 3,
      DSPERIODOORCA: I18n.translate('gop:TERCEIROBIMESTRE'),
      IDPERIODOORCA: 'BIMESTRAL',
    },
    {
      NRPERIODOORCA: 4,
      DSPERIODOORCA: I18n.translate('gop:QUARTOBIMESTRE'),
      IDPERIODOORCA: 'BIMESTRAL',
    },
    {
      NRPERIODOORCA: 5,
      DSPERIODOORCA: I18n.translate('gop:QUINTOBIMESTRE'),
      IDPERIODOORCA: 'BIMESTRAL',
    },
    {
      NRPERIODOORCA: 6,
      DSPERIODOORCA: I18n.translate('gop:SEXTOBIMESTRE'),
      IDPERIODOORCA: 'BIMESTRAL',
    },
    {
      NRPERIODOORCA: 1,
      DSPERIODOORCA: I18n.translate('gop:JANEIRO'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 2,
      DSPERIODOORCA: I18n.translate('gop:FEVEREIRO'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 3,
      DSPERIODOORCA: I18n.translate('gop:MARCO'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 4,
      DSPERIODOORCA: I18n.translate('gop:ABRIL'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 5,
      DSPERIODOORCA: I18n.translate('gop:MAIO'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 6,
      DSPERIODOORCA: I18n.translate('gop:JUNHO'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 7,
      DSPERIODOORCA: I18n.translate('gop:JULHO'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 8,
      DSPERIODOORCA: I18n.translate('gop:AGOSTO'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 9,
      DSPERIODOORCA: I18n.translate('gop:SETEMBRO'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 10,
      DSPERIODOORCA: I18n.translate('gop:OUTUBRO'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 11,
      DSPERIODOORCA: I18n.translate('gop:NOVEMBRO'),
      IDPERIODOORCA: 'MENSAL',
    },
    {
      NRPERIODOORCA: 12,
      DSPERIODOORCA: I18n.translate('gop:DEZEMBRO'),
      IDPERIODOORCA: 'MENSAL',
    },
  ];
  
  private defaultModal!: Modal;

  public async createDefaultModal(data) {
    this.defaultModal = ModalService.create({
      name: 'defaultModal',
      children: [
        {
          name: 'defaultModalHeader',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: data.TITLE,
              cssClass: 'zd-theme-font-title',
            },
          ],
          rightSlot: [
            {
              name: 'closeDefaultModalButton',
              component: 'ZdModalCloseButton',
              small: true,
              modalName: 'defaultModal',
            },
          ],
        },
        {
          name: 'defaultModalContainer',
          component: 'ZdContainer',
          scrollView: true,
          maxHeight: '300px',
          children: [
            {
              name: 'paragraph1',
              component: 'ZdText',
              text: data.MESSAGE,
              tag: 'p',
            },
          ],
        },
        {
          name: 'footerContainer',
          component: 'ZdContainer',
          children: [
            {
              name: 'exampleFooterFixed',
              component: 'ZdFooter',
              color: 'transparent',
              padless: true,
              rightSlot: [
                {
                  name: 'cancelButton',
                  component: 'ZdButton',
                  label: 'OK',
                  outline: true,
                  cssClass: 'zd-mr-2',
                  events: {
                    click: () => this.closeDefaultModal(),
                  },
                },
              ],
            },
          ],
        },
      ],
    });
  }

  public destroyDefaultModal() {
    this.defaultModal?.destroy();
  }

  public openDefaultModal() {
    this.defaultModal.show();
  }

  public closeDefaultModal() {
    this.defaultModal.hide();
  }
  
  /**
  * Returns all options referring to the parameterized IDPERIODOORCA
  *
  * @param {string} IDPERIODOORCA - Defines the type of parameterized period.
  *
  * @access public
  * @author Grazielle Antunes
  */
  dataPeriod(IDPERIODOORCA: string): any {
    const period = this.periodList.filter((item) => item.IDPERIODOORCA === IDPERIODOORCA);
    return period;
  }
  
  /**
  * Searches for the parameters registered in the database necessary to define the type of budget.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public async mountedParameters() {
    await BudgetParameters.getBudgetParameters();
  }
  
  /**
  * Default success message displayed when completing an operation.
  *
  * @param {string} words - optional phrase.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  successMessage(words: string = null): any {
    return AlertService.show({
      name: 'save',
      text: words ?? I18n.translate('gop::SUCCESS'),
      color: 'success',
      right: true,
      top: true,
      showDismiss: true,
    });
  }
  
  /**
  * Default error message displayed when an operation fails.
  *
  * @param {string} words - optional phrase.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  errorMessage(words: string = null): any {
    AlertService.show({
      name: 'save',
      text: I18n.translate(words) ?? I18n.translate('gop::ERROR'),
      color: 'error',
      right: true,
      top: true,
      showDismiss: true,
    });
  }
  
  /**
  * Default alert message displayed in an operation.
  *
  * @param {string} words - optional phrase.
  *
  * @access public
  * @author Grazielle Antunes
  */
  warningMessage(words: string): any {
    AlertService.show({
      name: 'save',
      text: I18n.translate(words),
      color: 'warning',
      right: true,
      top: true,
      showDismiss: true,
    });
  }
  
  /**
  * Open a modal on the screen.
  *
  * @param {function} actionConfirmed - method to be called after modal confirmation..
  * @param {object} data - data necessary to fill in the modal and to send the function passed in actionConfirmed.
  * @param {string} type - defines the type of modal to be opened, with OK being a model with an OK button
  * and YN being a model with a Yes and No button.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  /*
  public showDialog(actionConfirmed, data, type = 'OK') {
    if (type === 'YN' || type === 'YNA') {
      DialogService.show(this.createConfirmationDialog(actionConfirmed, data, type));
    } else {
      DialogService.show({
        name: 'observationDialogue',
        title: data.TITLE,
        text: data.MESSAGE,
        buttons: [
          {
            name: 'ok',
            component: 'ZdButton',
            label: 'OK',
            events: {
              click: () => DialogService.hide(),
            },
          },
        ],
      });
    }
  }
  */
  
  public async showDialog(actionConfirmed, data, type = 'OK'): Promise<string> {
    return new Promise((resolve) => {
      if (type === 'YN' || type === 'YNA') {
        DialogService.show(this.createConfirmationDialog(async (params, btn) => {
          await actionConfirmed(data, btn);
          resolve('Yes');
        }, data, type));
      } else {
        DialogService.show({
          name: 'observationDialogue',
          title: data.TITLE,
          text: data.MESSAGE,
          buttons: [
            {
              name: 'ok',
              component: 'ZdButton',
              label: 'OK',
              events: {
                click: () => {
                  DialogService.hide();
                  resolve('OK');
                },
              },
            },
          ],
        });
      }
    });
  }
  
  /**
  * Open a modal on the screen when the type is YN.
  *
  * @param {function} actionConfirmed - method to be called after modal confirmation..
  * @param {object} data - data necessary to fill in the modal and to send the function passed in actionConfirmed.
  * @param {string} type - defines the type of modal to be opened, with OK being a model with an OK button
  * and YN being a model with a Yes and No button.
  *
  * @access public
  * @author Grazielle Antunes
  */
  public createConfirmationDialog(actionConfirmed, data, type) {
    const dialog = {
      name: 'buttons-dialog',
      type: 'warning',
      title: I18n.translate('gop::CONFIRM_OPERATION'),
      text: data.MESSAGE,
      buttons: [
        {
          name: 'noConfirmation',
          component: 'ZdButton',
          label: 'Não',
          events: this.leaveDialogue,
          outline: true,
        },
        {
          name: 'yesConfirmation',
          component: 'ZdButton',
          label: 'Sim',
          events: {
            click: () => actionConfirmed(data, 'S'),
          },
        },
      ],
    };
    if (type === 'YNA') {
      dialog.buttons[0].events = { click: () => actionConfirmed(data, 'N') };
      const cancel = {
        name: 'cancel',
        component: 'ZdButton',
        label: 'Cancelar',
        events: this.leaveDialogue,
        outline: true,
      };
      dialog.buttons.push(cancel);
    }
    return dialog;
  }
  
  /**
  * Search for the parameterized IDPERIODOORCA and select it in the corresponding field.
  * Manipulates dependent fields by filling them with the options corresponding to the parameterization,
  * in addition to changing the visibility of the budget period field.
  *
  * @param {object} component - component that triggers the function.
  *
  * @access public
  * @author Grazielle Antunes
  */
  budgetPeriod({ component }) {
    const period = BudgetParameters.getParameters.IDPERIODOORCA;
    const textPeriod = I18n.translate(`gop::${period}`);
    
    if (component.name === 'NRPERIODOORCA' && period === 'ANUAL') {
      component.clearValidations();
    } else {
      component.addValidation('required');
    }
    if (component.name === 'IDPERIODOORCA') {
      component.value = I18n.translate(textPeriod);
      
      if (period === 'ANUAL') {
        this.periodSize = 10;
      } else {
        this.periodSize = 4;
      }
    } else if (period === 'ANUAL') {
      component.isVisible = false;
      component.validations.required = false;
      component.datasource.data = [];
    } else {
      component.datasource.data = this.dataPeriod(period);
    }
  }
  
  /**
  * Closes the modal.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public leaveDialogue: any = {
    click: () => DialogService.hide(),
  };
  
  /**
  * Closes the modal.
  *
  * @access public
  * @author Jefferson Gouvea
  */
  public hideDialogue() {
    DialogService.hide();
  }
  
  /**
  * Returns whether the value type is array.
  *
  *  @param {} value - value to be checked..
  *
  * @access public
  * @author Grazielle Antunes
  */
  isArray(value) {
    return Array.isArray(value);
  }
  
  /**
  * Returns whether the value is undefined.
  *
  *  @param {} value - value to be checked..
  *
  * @access public
  * @author Grazielle Antunes
  */
  isUndefined(value) {
    return typeof (value) === 'undefined';
  }
  
  /**
  * Returns whether the value type is null.
  *
  *  @param {} value - value to be checked..
  *
  * @access public
  * @author Grazielle Antunes
  */
  isNull(value) {
    return value === null;
  }
  
  /**
  * Returns whether the value is undefined or null.
  *
  *  @param {} value - value to be checked..
  *
  * @access public
  * @author Grazielle Antunes
  */
  isUndefinedOrNull(value) {
    return this.isUndefined(value) || this.isNull(value);
  }
  
  /**
  * Returns whether the value is null or blank.
  *
  *  @param {} value - value to be checked..
  *
  * @access public
  * @author Grazielle Antunes
  */
  public isEmptyOrBlank(value) {
    return this.isUndefinedOrNull(value) || value.toString().trim().length === 0;
  }
}
